// file      : xsd/cxx/parser/elements.cxx
// author    : Boris Kolpackov <boris@codesynthesis.com>
// copyright : Copyright (c) 2005-2007 Code Synthesis Tools CC
// license   : GNU GPL v2 + exceptions; see accompanying LICENSE file

#include <cxx/parser/elements.hxx>

namespace CXX
{
  namespace Parser
  {
    // Keep this symbol first to help HP-UX linker (long symbols?).
    //
    Content::Value Context::
    content (SemanticGraph::Complex& c)
    {
      using namespace SemanticGraph;

      if (c.mixed ())
        return Content::mixed;

      if (c.inherits_p ())
      {
        Type& base (c.inherits ().base ());

        if (Complex* cb = dynamic_cast<Complex*> (&base))
          return content (*cb);

        if (base.is_a<AnyType> ())
          return Content::complex;

        // Everyhting else (built-in type and AnySimpleType) is simple
        // content.
        //
        return Content::simple;
      }
      else
        return Content::complex;
    }

    Context::
    Context (std::wostream& o,
             SemanticGraph::Schema& root,
             CLI::Options const& ops)
        : CXX::Context (o,
                        root,
                        ops.value<CLI::char_type> (),
                        ops.value<CLI::include_with_brackets> (),
                        ops.value<CLI::include_prefix> (),
                        ops.value<CLI::export_symbol> (),
                        ops.value<CLI::namespace_map> (),
                        ops.value<CLI::namespace_regex> (),
                        ops.value<CLI::namespace_regex_trace> (),
                        ops.value<CLI::include_regex> (),
                        ops.value<CLI::include_regex_trace> (),
                        ops.value<CLI::generate_inline> ()),
          options (options_),
          std_string_type (std_string_type_),
          xml_parser (xml_parser_),
          simple_base (simple_base_),
          complex_base (complex_base_),
          parser_template (parser_template_),
          validation (validation_),
          options_ (ops),
          std_string_type_ (CXX::Context::string_type),
          xml_parser_ (ops.value<CLI::xml_parser> ()),
          parser_template_ (L"::xsd::cxx::parser::parser"),
          validation_ ((ops.value<CLI::xml_parser> () == "expat" ||
                        ops.value<CLI::generate_validation> ()) &&
                       !ops.value<CLI::suppress_validation> ())
    {
      string_type = L"::xsd::cxx::parser::ro_string< " +
        char_type + L" >";

      simple_base_ += L"::xsd::cxx::parser::";
      simple_base_ += (validation ? L"validating" : L"non_validating");
      simple_base_ += L"::simple_content< " + char_type + L" >";

      complex_base_ += L"::xsd::cxx::parser::";
      complex_base_ += (validation ? L"validating" : L"non_validating");
      complex_base_ += L"::complex_content< " + char_type + L" >";
    }
  }
}
